@@ -19,16 +19,10 @@
 import re
 from typing import Optional, Type
 
-from . import (
-    errors,
-    hooks,
-    registry,
-    urlutils,
-    )
+from . import errors, hooks, registry, urlutils
 
 
 class NoSuchProject(errors.BzrError):
-
     _fmt = "Project does not exist: %(project)s."
 
     def __init__(self, project):
@@ -37,7 +31,6 @@ class NoSuchProject(errors.BzrError):
 
 
 class MergeProposalExists(errors.BzrError):
-
     _fmt = "A merge proposal already exists: %(url)s."
 
     def __init__(self, url, existing_proposal=None):
@@ -47,7 +40,6 @@ class MergeProposalExists(errors.BzrError):
 
 
 class UnsupportedForge(errors.BzrError):
-
     _fmt = "No supported forge for %(branch)s."
 
     def __init__(self, branch):
@@ -56,26 +48,32 @@ class UnsupportedForge(errors.BzrError):
 
 
 class ReopenFailed(errors.BzrError):
-
     _fmt = "Reopening the merge proposal failed: %(error)s."
 
 
 class TitleUnsupported(errors.BzrError):
-
     _fmt = "The merge proposal %(mp)s does not support a title."
 
 
+class AutoMergeUnsupported(errors.BzrError):
+    _fmt = "The merge proposal %(mp)s does not support automerge."
+
+
 class ProposeMergeHooks(hooks.Hooks):
     """Hooks for proposing a merge on Launchpad."""
 
     def __init__(self):
         hooks.Hooks.__init__(self, __name__, "Proposer.hooks")
         self.add_hook(
-            'get_prerequisite',
-            "Return the prerequisite branch for proposing as merge.", (3, 0))
+            "get_prerequisite",
+            "Return the prerequisite branch for proposing as merge.",
+            (3, 0),
+        )
         self.add_hook(
-            'merge_proposal_body',
-            "Return an initial body for the merge proposal message.", (3, 0))
+            "merge_proposal_body",
+            "Return an initial body for the merge proposal message.",
+            (3, 0),
+        )
 
 
 class LabelsUnsupported(errors.BzrError):
@@ -99,7 +97,7 @@ class PrerequisiteBranchUnsupported(errors.BzrError):
 class ForgeLoginRequired(errors.BzrError):
     """Action requires forge login credentials."""
 
-    _fmt = "Action requires credentials for hosting site %(forge)r."""
+    _fmt = "Action requires credentials for hosting site %(forge)r." ""
 
     def __init__(self, forge):
         errors.BzrError.__init__(self)
@@ -109,24 +107,31 @@ class ForgeLoginRequired(errors.BzrError):
 class SourceNotDerivedFromTarget(errors.BzrError):
     """Source branch is not derived from target branch."""
 
-    _fmt = ("Source %(source_branch)r not derived from "
-            "target %(target_branch)r.")
+    _fmt = "Source %(source_branch)r not derived from " "target %(target_branch)r."
 
     def __init__(self, source_branch, target_branch):
         errors.BzrError.__init__(
-            self, source_branch=source_branch,
-            target_branch=target_branch)
+            self, source_branch=source_branch, target_branch=target_branch
+        )
 
 
-class MergeProposal(object):
+class MergeProposal:
     """A merge proposal.
 
     :ivar url: URL for the merge proposal
     """
 
+    supports_auto_merge: bool
+
     def __init__(self, url=None):
         self.url = url
 
+    def __str__(self):
+        return self.url
+
+    def __repr__(self):
+        return "<%s(%r)>" % (self.__class__.__name__, self.url)
+
     def get_web_url(self):
         raise NotImplementedError(self.get_web_url)
 
@@ -154,7 +159,7 @@ class MergeProposal(object):
         """Set the propose commit message."""
         raise NotImplementedError(self.set_commit_message)
 
-    def get_source_branch_url(self):
+    def get_source_branch_url(self, *, preferred_schemes=None):
         """Return the source branch."""
         raise NotImplementedError(self.get_source_branch_url)
 
@@ -162,7 +167,7 @@ class MergeProposal(object):
         """Return the latest revision for the source branch."""
         raise NotImplementedError(self.get_source_revision)
 
-    def get_target_branch_url(self):
+    def get_target_branch_url(self, *, preferred_schemes=None):
         """Return the target branch."""
         raise NotImplementedError(self.get_target_branch_url)
 
@@ -191,7 +196,7 @@ class MergeProposal(object):
         """
         raise NotImplementedError(self.is_closed)
 
-    def merge(self, commit_message=None):
+    def merge(self, commit_message=None, auto=False):
         """Merge this merge proposal."""
         raise NotImplementedError(self.merge)
 
@@ -203,13 +208,11 @@ class MergeProposal(object):
         raise NotImplementedError(self.can_be_merged)
 
     def get_merged_by(self):
-        """If this proposal was merged, who merged it.
-        """
+        """If this proposal was merged, who merged it."""
         raise NotImplementedError(self.get_merged_by)
 
     def get_merged_at(self):
-        """If this proposal was merged, when it was merged.
-        """
+        """If this proposal was merged, when it was merged."""
         raise NotImplementedError(self.get_merged_at)
 
     def post_comment(self, body):
@@ -225,7 +228,7 @@ class MergeProposal(object):
         raise NotImplementedError(self.reopen)
 
 
-class MergeProposalBuilder(object):
+class MergeProposalBuilder:
     """Merge proposal creator.
 
     :param source_branch: Branch to propose for merging
@@ -246,14 +249,21 @@ class MergeProposalBuilder(object):
         raise NotImplementedError(self.get_initial_body)
 
     def get_infotext(self):
-        """Determine the initial comment for the merge proposal.
-        """
+        """Determine the initial comment for the merge proposal."""
         raise NotImplementedError(self.get_infotext)
 
-    def create_proposal(self, description, title=None, reviewers=None,
-                        labels=None, prerequisite_branch=None,
-                        commit_message=None,
-                        work_in_progress=False, allow_collaboration=False):
+    def create_proposal(
+        self,
+        description,
+        title=None,
+        reviewers=None,
+        labels=None,
+        prerequisite_branch=None,
+        commit_message=None,
+        work_in_progress=False,
+        allow_collaboration=False,
+        delete_source_after_merge: Optional[bool] = None,
+    ):
         """Create a proposal to merge a branch for merging.
 
         Args:
@@ -267,14 +277,15 @@ class MergeProposalBuilder(object):
           allow_collaboration:
             Whether to allow changes to the branch from the target branch
             maintainer(s)
+          delete_after_merge: Whether to delete the source branch after it
+            has been merged
         Returns: A `MergeProposal` object
         """
         raise NotImplementedError(self.create_proposal)
 
 
-class Forge(object):
-    """A hosting site manager.
-    """
+class Forge:
+    """A hosting site manager."""
 
     # Does this forge support arbitrary labels being attached to merge
     # proposals?
@@ -285,7 +296,7 @@ class Forge(object):
     @property
     def name(self):
         """Name of this instance."""
-        return "%s at %s" % (type(self).__name__, self.base_url)
+        return "{} at {}".format(type(self).__name__, self.base_url)
 
     # Does this forge support suggesting a commit message in the
     # merge proposal?
@@ -302,9 +313,18 @@ class Forge(object):
     # Does this forge support the allow_collaboration flag?
     supports_allow_collaboration: bool = False
 
-    def publish_derived(self, new_branch, base_branch, name, project=None,
-                        owner=None, revision_id=None, overwrite=False,
-                        allow_lossy=True, tag_selector=None):
+    def publish_derived(
+        self,
+        new_branch,
+        base_branch,
+        name,
+        project=None,
+        owner=None,
+        revision_id=None,
+        overwrite=False,
+        allow_lossy=True,
+        tag_selector=None,
+    ):
         """Publish a branch to the site, derived from base_branch.
 
         :param base_branch: branch to derive the new branch from
@@ -315,9 +335,10 @@ class Forge(object):
         """
         raise NotImplementedError(self.publish_derived)
 
-    def get_derived_branch(self, base_branch, name, project=None, owner=None, preferred_schemes=None):
-        """Get a derived branch ('a fork').
-        """
+    def get_derived_branch(
+        self, base_branch, name, project=None, owner=None, preferred_schemes=None
+    ):
+        """Get a derived branch ('a fork')."""
         raise NotImplementedError(self.get_derived_branch)
 
     def get_push_url(self, branch):
@@ -339,7 +360,7 @@ class Forge(object):
         """
         raise NotImplementedError(self.get_proposer)
 
-    def iter_proposals(self, source_branch, target_branch, status='open'):
+    def iter_proposals(self, source_branch, target_branch, status="open"):
         """Get the merge proposals for a specified branch tuple.
 
         :param source_branch: Source branch
@@ -364,24 +385,24 @@ class Forge(object):
 
     @classmethod
     def probe_from_hostname(cls, hostname, possible_transports=None):
-        """Create a Forge object if this forge knows about a hostname.
-        """
+        """Create a Forge object if this forge knows about a hostname."""
         raise NotImplementedError(cls.probe_from_hostname)
 
     @classmethod
     def probe_from_branch(cls, branch):
         """Create a Forge object if this forge knows about a branch."""
         url = urlutils.strip_segment_parameters(branch.user_url)
-        return cls.probe_from_url(
-            url, possible_transports=[branch.control_transport])
+        return cls.probe_from_url(url, possible_transports=[branch.control_transport])
 
     @classmethod
     def probe_from_url(cls, url, possible_transports=None):
         """Create a Forge object if this forge knows about a URL."""
         hostname = urlutils.URL.from_string(url).host
-        return cls.probe_from_hostname(hostname, possible_transports=possible_transports)
+        return cls.probe_from_hostname(
+            hostname, possible_transports=possible_transports
+        )
 
-    def iter_my_proposals(self, status='open', author=None):
+    def iter_my_proposals(self, status="open", author=None):
         """Iterate over the proposals created by the currently logged in user.
 
         :param status: Only yield proposals with this status
@@ -402,13 +423,11 @@ class Forge(object):
         raise NotImplementedError(self.iter_my_forks)
 
     def delete_project(self, name):
-        """Delete a project.
-        """
+        """Delete a project."""
         raise NotImplementedError(self.delete_project)
 
-    def create_project(self, name):
-        """Create a project.
-        """
+    def create_project(self, name, summary=None):
+        """Create a project."""
         raise NotImplementedError(self.create_project)
 
     @classmethod
@@ -439,9 +458,9 @@ def determine_title(description):
     else:
         raise ValueError
     try:
-        i = firstline.index('. ')
+        i = firstline.index(". ")
     except ValueError:
-        return firstline.rstrip('.')
+        return firstline.rstrip(".")
     else:
         return firstline[:i]
 
@@ -471,8 +490,7 @@ def get_forge(branch, possible_forges=None):
 
 
 def get_forge_by_hostname(hostname: str):
-    """Get a forge from a hostname.
-    """
+    """Get a forge from a hostname."""
     for instance in iter_forge_instances():
         try:
             return instance.probe_from_hostname(hostname)
@@ -491,8 +509,7 @@ def iter_forge_instances(forge: Optional[Type[Forge]] = None):
     else:
         forge_clses = [forge]
     for forge_cls in forge_clses:
-        for instance in forge_cls.iter_instances():
-            yield instance
+        yield from forge_cls.iter_instances()
 
 
 def get_proposal_by_url(url):
@@ -510,7 +527,7 @@ def get_proposal_by_url(url):
     raise UnsupportedForge(url)
 
 
-def create_project(url: str) -> None:
+def create_project(url: str, *, summary=None) -> None:
     """Create a project.
 
     Args:
@@ -524,7 +541,7 @@ def create_project(url: str) -> None:
         except UnsupportedForge:
             pass
         else:
-            hoster.create_project(parsed_url.path)
+            hoster.create_project(parsed_url.path, summary=summary)
             break
     else:
         raise UnsupportedForge(url)
